@@ -1,5 +1,5 @@
 /* client.c --- Non-standard SASL mechanism LOGIN, client side.
- * Copyright (C) 2002, 2003, 2004  Simon Josefsson
+ * Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009  Simon Josefsson
  *
  * This file is part of GNU SASL Library.
  *
@@ -20,7 +20,7 @@
  *
  */
 
-#if HAVE_CONFIG_H
+#ifdef HAVE_CONFIG_H
 # include "config.h"
 #endif
 
@@ -103,5 +103,8 @@ _gsasl_login_client_finish (Gsasl_session * sctx, void *mech_data)
 {
   struct _Gsasl_login_client_state *state = mech_data;
 
+  if (!state)
+    return;
+
   free (state);
 }
